from .lit import geezlit


def main():
    import argparse

    parser = argparse.ArgumentParser("GeezLit")
    parser.add_argument("geez", type=str, help="Geez text to convert to latex encoding!")
    args = parser.parse_args()

    if args.geez:
        res = geezlit(args.geez)
        print(res)
    else:
        print("!! Provide text to convert with `-g` !!")


if "__main__" == __name__:
    main()



from .style.base import BaseLit
from .style.ethiop import EthiopLit


def get_liter(name):
    lit_classes = {BaseLit, EthiopLit}
    for cls in lit_classes:
        if name == cls.NAME:
            return cls()
    raise TypeError("Unkown GeezLit style: " + name)


def geezlit(text, style="ethiop"):
    lit = get_liter(name=style)
    return lit.to_ascii(text)



class BaseLit(object):
    """Base Ge'ez transliteration class"""

    NAME = "base"

    def __init__(self):
        self.he_int = 4608
        self.po_int = 4950
        self.ascii_rows = list(range(47))
        self.ascii_cols = list(range(8))
        self.ascii_cols_labial = list(range(8))
        self.geez_rows_labial = {9, 11, 17, 22, 24, 34, 36}
        self.geez_puncs = dict()

    def char_is_geez(self, char):
        """
        char must bee of length 1,
        else this will throw TypeError
        """
        return self.he_int <= ord(char) <= self.po_int

    def get_geez_row_col(self, char):
        cint = ord(char)
        assert self.he_int <= cint <= self.po_int
        col = cint % 8
        row = ((cint - col) - self.he_int) // 8
        return row, col

    def to_ascii(self, text):
        result = list()
        for char in text:
            try:
                if not self.char_is_geez(char):
                    result.append(self.geez_puncs.get(char, char))
                    continue
                row, col = self.get_geez_row_col(char)
                rtex = self.ascii_rows[row]
                if row in self.geez_rows_labial:
                    ctex = self.ascii_cols_labial[col]
                else:
                    ctex = self.ascii_cols[col]
                result.append(f"{rtex}{ctex}")
            except Exception as ex:
                print(f"Couldn't convert: '{char}'! {ex}")
                result.append(":|:")
        return "".join(result)



from geezlit.style.base import BaseLit


class GeezIMELit(BaseLit):
    """Ge'ez transliteration based on the GeezIME keyboard"""

    NAME = "geezime"

    def __init__(self):
        super().__init__()



from geezlit.style.base import BaseLit


class MSERALit(BaseLit):
    """Ge'ez transliteration based modified SERA in HornMorpho"""

    NAME = "msera"

    def __init__(self):
        super().__init__()



from setuptools import find_packages, setup


def read_requirements(filename: str):
    with open(filename) as requirements_file:
        import re

        def fix_url_dependencies(req: str) -> str:
            """Pip and setuptools disagree about how URL dependencies should be handled."""
            m = re.match(
                r"^(git\+)?(https|ssh)://(git@)?github\.com/([\w-]+)/(?P<name>[\w-]+)\.git", req
            )
            if m is None:
                return req
            else:
                return f"{m.group('name')} @ {req}"

        requirements = []
        for line in requirements_file:
            line = line.strip()
            if line.startswith("#") or len(line) <= 0:
                continue
            requirements.append(fix_url_dependencies(line))
    return requirements


app_info = {}  # type: ignore
with open("geezlit/version.py", "r") as version_file:
    exec(version_file.read(), app_info)

setup(
    name=app_info["__title__"],
    version=app_info["__version__"],
    description=app_info["__description__"],
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url=app_info["__url__"],
    author=app_info["__author__"],
    author_email=app_info["__author_email__"],
    license=app_info["__license__"],
    packages=find_packages(
        exclude=["*.tests", "*.tests.*", "tests.*", "tests"],
    ),
    package_data={"geezlit": ["py.typed"]},
    install_requires=read_requirements("requirements.txt"),
    extras_require={"dev": read_requirements("dev-requirements.txt")},
    python_requires=">=3.5",
    entry_points={
        "console_scripts": ["geezlit=geezlit.cli:main"],
    },
    classifiers=[
        "Intended Audience :: Science/Research",
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    keywords="geez, transliteration",
)



_MAJOR = "0"
_MINOR = "1"
# On main and in a nightly release the patch should be one ahead of the last
# released build.
_PATCH = "2"
# This is mainly for nightly builds which have the suffix ".dev$DATE". See
# https://semver.org/#is-v123-a-semantic-version for the semantics.
_SUFFIX = ""

VERSION_SHORT = "{0}.{1}".format(_MAJOR, _MINOR)
VERSION = "{0}.{1}.{2}{3}".format(_MAJOR, _MINOR, _PATCH, _SUFFIX)


__title__ = "geezlit"
__version__ = VERSION
__description__ = "Ge'ez Transliteration"
__author__ = "Fitsum Gaim"
__author_email__ = "fitsum@geezlab.com"
__copyright__ = "2022 Fitsum Gaim"
__license__ = "Apache 2.0"
__url__ = "https://github.com/fgaim/geezlit"
